part of tecfy_corev2_client_package;

class LocationPropertyComponent extends StatefulWidget {
  final Key? key;

  /// ini value
  final Map<String, dynamic>? initialValue;

  /// label string
  final String? label;
  final String? description;

  /// is requried
  final bool isRequired;

  /// is read only
  final bool readOnly;

  /// is add new
  final bool? isAddNew;

  /// on changed
  final void Function(Map<String, dynamic>)? onChanged;

  LocationPropertyComponent(
      {this.initialValue,
      this.label,
      this.key,
      this.isAddNew,
      this.description,
      this.isRequired = false,
      this.onChanged,
      this.readOnly = false});

  @override
  _LocationPropertyComponentState createState() =>
      _LocationPropertyComponentState();
}

class _LocationPropertyComponentState extends State<LocationPropertyComponent> {
  LatLng? _point;

  @override
  initState() {
    if (widget.initialValue == null || widget.initialValue!.isEmpty) {
      _point = null;
    } else {
      _point = LatLng(BaseModel.getDoubleEx(widget.initialValue!['lat'])!,
          BaseModel.getDoubleEx(widget.initialValue!['lng'])!);
    }

    super.initState();
    if (widget.isAddNew == true) {
      if (_point == null) {
        WidgetsBinding.instance.addPostFrameCallback((timeStamp) async {
          await _getCurrentLocation();
        });
      }
    }
  }

  Future<void> _getCurrentLocation() async {
    Location _location = Location();
    bool serviceEnabled;
    PermissionStatus permissionGranted;

    // Check if location services are enabled
    serviceEnabled = await _location.serviceEnabled();
    if (!serviceEnabled) {
      try {
        AppNavigator.showMessage(
            context,
            AppLocale.translate(
                "To access your Location, please activate Location services"),
            MessageType.info);
        serviceEnabled = await _location.requestService();
        if (!serviceEnabled) {
          // Location services are disabled
          await AppNavigator.showMessage(
              context,
              AppLocale.translate("Location services are inActive"),
              MessageType.warning);
          return;
        }
      } catch (ex) {
        await AppNavigator.showMessage(
            context,
            AppLocale.translate(
                "To access your Location, please activate Location services and reload the page"),
            MessageType.info);
        return;
      }
    }

    // Check if the app has permission to access location
    permissionGranted = await _location.hasPermission();
    if (permissionGranted == PermissionStatus.denied) {
      try {
        permissionGranted = await _location.requestPermission();
        if (permissionGranted != PermissionStatus.granted) {
          // Location permission denied
          await AppNavigator.showMessage(
              context,
              AppLocale.translate("Location services are Denied"),
              MessageType.warning);
          return;
        }
      } catch (ex) {
        await AppNavigator.showMessage(
            context,
            AppLocale.translate(
                "'To access your Location, please activate Location services and reload the page'"),
            MessageType.info);
        return;
      }
    }

    // Get the current location
    LocationData locationData = await _location.getLocation();
    double? latitude = locationData.latitude;
    double? longitude = locationData.longitude;
    if (latitude != null || longitude != null) {
      _point = LatLng(latitude!, longitude!);
    }
    setState(() {});
    debugPrint(
        'Latitude: ${latitude.toString()}, Longitude: ${longitude.toString()}');
  }

  _onChanged() {
    setState(() {});
    widget.onChanged
        ?.call({'lat': _point?.latitude ?? 0, 'lng': _point?.longitude ?? 0});
  }

  @override
  Widget build(BuildContext context) {
    return MaterialButton(
        padding: EdgeInsets.all(0),
        onPressed: () async {
          var result = await showDialog(
              context: context,
              builder: (c) =>
                  LocationSelectPage(_point ?? LatLng(31.274750, 32.301204)));
          if (result != null && result is LatLng) {
            _point = result;
            _onChanged();
          }
        },
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            SizedBox(height: 10),
            Tooltip(
                message: widget.description ?? '',
                child: TecfyText(
                  '${widget.label ?? ''}${widget.isRequired ? '*' : ''}',
                  style: ThemeService.labelStyle,
                )),
            TecfyText((_point == null ||
                    _point?.latitude == null ||
                    _point?.longitude == null)
                ? '[Click To Select]'
                : '${_point?.latitude ?? ''}  , ${_point?.longitude ?? ""}'),
            SizedBox(height: 10),
            Container(height: 1, color: ThemeService.textSubColor)
          ],
        ));
  }
}
