part of tecfy_corev2_client_package;

class ReportListPage extends StatefulWidget {
  /// if true only body shown without BasePage
  bool isLite;

  /// isDropDown will always return a button when you click on it dropdown will be shown
  bool isDropDown;

  /// should be used if isDropDown is true
  BuildContext? context;

  /// code send to api to get list of reports
  String code;
  ReportListPage(
      {this.isLite = false,
      this.isDropDown = false,
      this.code = "AREP",
      this.context}) {
    if (this.isDropDown) {
      this.isLite = true;
    }
  }
  @override
  _ReportListPageState createState() => _ReportListPageState();
}

class _ReportListPageState extends State<ReportListPage> {
  List<dynamic>? _reports;

  @override
  void initState() {
    if (!widget.isDropDown) {
      _getData();
    }
    super.initState();
  }

  _getData() async {
    try {
      var result = await TecfyBasicApp.apiService.httpGetDynamic(
          getUrlEx('/api/report/setup/reports/${widget.code}')!);
      if (result.success) {
        if (TecfyBasicApp.isNewer(result.lastVersion)) {
          AppNavigator.navigateTo(
              context, () => UpgradePage(isRequired: false));
        }
        setState(() {
          _reports = result.data;
        });
      }
    } catch (e) {
      print("Exception: $e");
    }
  }

  _onPressed(report) async {
    if (widget.isDropDown) Navigator.pop(context);
    await ReportDataService.onOpenReport(context, report, null, widget.isLite);
  }

  Offset _tapPosition = Offset.zero;
  void _getTapPosition(TapDownDetails details) {
    final RenderBox referenceBox =
        widget.context!.findRenderObject() as RenderBox;
    setState(() {
      _tapPosition = referenceBox.globalToLocal(details.globalPosition);
    });
  }

  void _showMenu() async {
    final RenderObject? overlay =
        Overlay.of(context).context.findRenderObject();

    final result = await showMenu(
        context: context,

        // Show the context menu at the tap location
        position: RelativeRect.fromLTRB(_tapPosition.dx, _tapPosition.dy, 0, 0),

        // set a list of choices for the context menu
        items: (_reports == null || (_reports?.isEmpty ?? false))
            ? [
                PopupMenuItem(
                  child: SizedBox(),
                )
              ]
            : _reports
                    ?.map<PopupMenuItem>(
                      (report) => PopupMenuItem(
                        child: itemRowWidget(report),
                      ),
                    )
                    .toList() ??
                []);
  }

  @override
  Widget build(BuildContext context) {
    if (widget.isDropDown)
      return GestureDetector(
        onTapDown: (details) async {
          _getTapPosition(details);
          await _getData();
          _showMenu();
        },
        child: Padding(
          padding: const EdgeInsets.all(8.0),
          child: MouseRegion(
              cursor: SystemMouseCursors.click,
              child: Icon(Icons.text_snippet)),
        ),
      );

    if (widget.isLite) return childWidget;

    return BasePage(context, 'Reports', 'report',
        isMobile: LayoutService.isMobile(context), child: childWidget);
  }

  Widget get childWidget => _reports == null
      ? Center(child: AppNavigator.getLoadingWidget(100))
      : ListView(
          children: _reports
                  ?.map<Widget>(
                    (report) => Container(
                      alignment: Alignment.center,
                      child: Container(
                        height: 50,
                        constraints: BoxConstraints(maxWidth: 300),
                        padding: EdgeInsets.all(5),
                        child: itemRowWidget(report),
                      ),
                    ),
                  )
                  .toList() ??
              [],
        );

  Widget itemRowWidget(dynamic report) => MaterialButton(
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(5)),
      padding: EdgeInsets.all(5),
      color: ThemeService.mainColor,
      onPressed: () => _onPressed(report),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.start,
        children: <Widget>[
          SizedBox(width: 5),
          // isBussy
          //     ? Transform.scale(
          //         scale: 0.5,
          //         child: CircularProgressIndicator(
          //           valueColor: AlwaysStoppedAnimation<Color>(
          //               ThemeService.textOnMainColor),
          //         ))
          //     :
          Transform.scale(
              scale: 0.8, child: Icon(Icons.print, color: Colors.white)),
          SizedBox(width: 5),
          Text(
            (MapService.getValue(report, 'name') as Map<String, dynamic>)[
                    LocalConfigService.languagePrefix.toLowerCase()] ??
                'N/A',
            style: TextStyle(color: ThemeService.textOnMainColor),
          ),
        ],
      ));
}
